/**
 * This class provides the methods and variables needed to create instances
 * of courses, whether for a course catalog or other uses.
 * @author Patrick Kelley
 * 
 * 3/8/2008 - Added methods to override equals() - pek
 */

package CourseCatalog;

import java.util.zip.*;

public class CourseInfo implements Comparable {
    /** Class variables */
    private String  Department;
    private String  CourseNum;
    private String  Description;
    private String  Instructor;
    private int     Credit;
    
    /** Constructors */
    /**
     * This constructor takes no arguments and creates an empty course
     */
    public CourseInfo (){
        Department = "";
        CourseNum = "";
        Description = "";
        Instructor = "";
        Credit = 0;
    }
    
    /**
     * This constructor takes passed arguments and creates a course
     */
    public CourseInfo (String dept, String num, String desc, String inst, int cred){
        Department = dept;
        CourseNum = num;
        Description = desc;
        Instructor = inst;
        Credit = cred;        
    }
    
    /** Accessors */
    /**
     * This accessor returns the Department Code
     */
    public String getDept (){
        return Department;
    }
    
    /**
     * This accessor returns the Course Number
     */
    public String getNum (){
        return CourseNum;
    }
    
    /**
     * This accessor returns the Description
     */
    public String getDesc (){
        return Description;
    }
    
    /**
     * This accessor returns the Instructor
     */
    public String getProf (){
        return Instructor;
    }
    
    /**
     * This accessor returns the Course Credit
     */
    public int getCredit (){
        return Credit;
    }
    
    /** Mutators */
    /**
     * This mutator lets you change the Department Code
     */
    public void setDept (String dept){
        Department = dept;
    }
    
    /**
     * This mutator lets you change the Course Number
     */
    public void setNum (String num){
        CourseNum = num;
    }
    
    /** 
     * This mutator lets you change the Description
     */
    public void setDesc (String desc){
        Description = desc;
    }
    
    /**
     * This mutator lets you change the Instructor
     */
    public void setProf (String inst){
        Instructor = inst;
    }
    
    /**
     * This mutator lets you set the Course Credits
     */
    public void setCredits (int cred){
        Credit = cred;
    }

    /** Other methods */
    /**
     * compareTo implements the comparable abstract method that will
     * allow you to sort objects of this class.  Basically, it lets you
     * order an object in relation to another
     * @param o is the other object to be compared
     * @return returns -1 if this object is greater, 0 if they are equal,
     *         and 1 if this object is smaller.
     */
    public int compareTo(Object o) {
        if (getClass() == o.getClass())
        {
            if (Department.compareTo(((CourseInfo) o).Department) !=0)
                return Department.compareTo(((CourseInfo) o).Department);
            else
                return CourseNum.compareTo(((CourseInfo) o).CourseNum);
        }
        System.out.println("Incompatible class types");
        return 0;
    }    

    /**
     * equals implements the comparable abstract method that will
     * allow you to determine if two objects are equivalent
     * @param o is the other object to be compared
     * @return true if they are equivalent, else false.
     */
    @Override
    public boolean equals(Object o) {
        if (getClass() == o.getClass())
        {
            if (!Department.equals(((CourseInfo)o).Department))
                return false;
            if (!CourseNum.equals(((CourseInfo)o).CourseNum))
                return false;
            if (!Description.equals(((CourseInfo)o).Description))
                return false;
            if (!Instructor.equals(((CourseInfo)o).Instructor))
                return false;
            if (Credit != ((CourseInfo)o).Credit)
                return false;
            return true;
        }
        return false;
    }

    /** 
     * Because we override equals(), we should also override the hashcode method
     */
    @Override
    public int hashCode() {
        int hash = 5;
        hash = 13 * hash + (this.Department != null ? this.Department.hashCode() : 0);
        hash = 13 * hash + (this.CourseNum != null ? this.CourseNum.hashCode() : 0);
        hash = 13 * hash + (this.Description != null ? this.Description.hashCode() : 0);
        hash = 13 * hash + (this.Instructor != null ? this.Instructor.hashCode() : 0);
        hash = 13 * hash + this.Credit;
        return hash;
    }
}